SystemOrganization addCategory: #'ToolBuilder-SUnit'!

ToolBuilder subclass: #SUnitToolBuilder
	instanceVariableNames: 'widgets'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolBuilder-SUnit'!

!SUnitToolBuilder commentStamp: 'cwp 6/7/2005 00:53' prior: 0!
I create a set of "stub" widgets that are useful for testing. Instead of drawing themselves in some GUI, they simulate graphical widgets for testing purposes. Through my widgets, unit tests can simulate user actions and make assertions about the state of the display.

See TestRunnerPlusTest for examples.!

!SUnitToolBuilder methodsFor: 'building' stamp: 'ar 7/14/2005 22:33'!
buildPluggableButton: aSpec
	| w |
	w := ButtonStub fromSpec: aSpec.
	self register: w id: aSpec name.
	^w! !

!SUnitToolBuilder methodsFor: 'building' stamp: 'ar 7/14/2005 22:33'!
buildPluggableList: aSpec 
	| w |
	w := ListStub fromSpec: aSpec.
	self register: w id: aSpec name.
	^w! !

!SUnitToolBuilder methodsFor: 'building' stamp: 'cwp 6/8/2005 23:34'!
buildPluggableMenu: aSpec 
	^ MenuStub fromSpec: aSpec! !

!SUnitToolBuilder methodsFor: 'building' stamp: 'ar 7/14/2005 22:33'!
buildPluggablePanel: aSpec
	| w |
	w := PanelStub fromSpec: aSpec.
	self register: w id: aSpec name.
	^w! !

!SUnitToolBuilder methodsFor: 'building' stamp: 'ar 7/14/2005 22:33'!
buildPluggableText: aSpec 
	| w |
	w := TextStub fromSpec: aSpec.
	self register: w id: aSpec name.
	^w! !

!SUnitToolBuilder methodsFor: 'building' stamp: 'ar 7/14/2005 22:33'!
buildPluggableTree: aSpec
	| w |
	w := TreeStub fromSpec: aSpec.
	self register: w id: aSpec name.
	^w! !

!SUnitToolBuilder methodsFor: 'building' stamp: 'stephaneducasse 2/3/2006 22:32'!
buildPluggableWindow: aSpec
	| window children |
	window := WindowStub fromSpec: aSpec.
	children := aSpec children.
	children isSymbol 
		ifFalse: [window children: (children collect: [:ea | ea buildWith: self])].
	self register: window id: aSpec name.
	^ window! !

!SUnitToolBuilder methodsFor: 'opening' stamp: 'cwp 7/14/2006 10:53'!
close: aWidget
	aWidget close! !

!SUnitToolBuilder methodsFor: 'opening' stamp: 'cwp 7/14/2006 10:48'!
open: anObject
	^ self build: anObject! !

!SUnitToolBuilder methodsFor: 'private' stamp: 'ar 7/14/2005 22:32'!
register: widget id: id
	id ifNil:[^self].
	widgets ifNil:[widgets := Dictionary new].
	widgets at: id put: widget.! !

!SUnitToolBuilder methodsFor: 'private' stamp: 'ar 7/14/2005 22:32'!
widgetAt: id ifAbsent: aBlock
	widgets ifNil:[^aBlock value].
	^widgets at: id ifAbsent: aBlock! !

Object subclass: #WidgetStub
	instanceVariableNames: 'spec'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolBuilder-SUnit'!

WidgetStub subclass: #ButtonStub
	instanceVariableNames: 'enabled'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolBuilder-SUnit'!

!ButtonStub methodsFor: 'simulating' stamp: 'stephaneducasse 2/3/2006 22:32'!
click
	| action |
	action := spec action.
	action isSymbol
		ifTrue: [self model perform: action]
		ifFalse: [action value]! !

!ButtonStub methodsFor: 'simulating' stamp: 'cwp 4/22/2005 22:44'!
color
	^ self model perform: spec color! !

!ButtonStub methodsFor: 'events' stamp: 'cwp 4/22/2005 22:01'!
eventAccessors
	^ #(label color state enabled)! !

!ButtonStub methodsFor: 'simulating' stamp: 'cwp 7/14/2006 11:09'!
isEnabled
	enabled ifNil: [enabled := spec model perform: spec enabled].
	^ enabled! !

WidgetStub subclass: #CompositeStub
	instanceVariableNames: 'children'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolBuilder-SUnit'!

!CompositeStub methodsFor: 'accessing' stamp: 'cwp 4/25/2005 03:50'!
children
	^children! !

!CompositeStub methodsFor: 'accessing' stamp: 'cwp 4/25/2005 03:50'!
children: anObject
	children := anObject! !

!CompositeStub methodsFor: 'accessing' stamp: 'cwp 4/25/2005 03:51'!
eventAccessors
	^ #(children)! !

!CompositeStub methodsFor: 'accessing' stamp: 'marcus.denker 11/10/2008 10:04'!
widgetNamed: aString
	self name = aString
		ifTrue: [^ self]
		ifFalse: [children do: [:ea | (ea widgetNamed: aString) ifNotNil: [:w | ^ w]]].
	^ nil! !

CompositeStub subclass: #PanelStub
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolBuilder-SUnit'!

CompositeStub subclass: #WindowStub
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolBuilder-SUnit'!

!WindowStub methodsFor: 'simulating' stamp: 'cwp 7/14/2006 10:58'!
close
	spec model perform: spec closeAction! !

!WindowStub methodsFor: 'events' stamp: 'cwp 4/25/2005 03:50'!
eventAccessors
	^ super eventAccessors, #(label)! !

WidgetStub subclass: #ListStub
	instanceVariableNames: 'list index'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolBuilder-SUnit'!

!ListStub methodsFor: 'simulating' stamp: 'cwp 5/27/2005 08:16'!
click: aString
	self clickItemAt: (self list indexOf: aString)! !

!ListStub methodsFor: 'simulating' stamp: 'stephaneducasse 2/3/2006 22:32'!
clickItemAt: anInteger
	| selector |
	selector := spec setIndex.
	selector
		ifNil: [self model perform: spec setSelected with: (self list at: anInteger)]
		ifNotNil: [self model perform: selector with: anInteger]
! !

!ListStub methodsFor: 'events' stamp: 'cwp 4/22/2005 22:35'!
eventAccessors
	^ #(list getIndex setIndex getSelected setSelected menu keyPress autoDeselect)! !

!ListStub methodsFor: 'simulating' stamp: 'cwp 5/26/2005 08:39'!
list
	^ list ifNil: [Array new]! !

!ListStub methodsFor: 'simulating' stamp: 'cwp 6/9/2005 08:15'!
menu
	^ MenuStub fromSpec:
		(self model 
			perform: spec menu 
			with: (PluggableMenuSpec withModel: self model))! !

!ListStub methodsFor: 'events' stamp: 'cwp 5/26/2005 08:39'!
refresh
	self refreshList.
	self refreshIndex! !

!ListStub methodsFor: 'events' stamp: 'stephaneducasse 2/3/2006 22:32'!
refreshIndex
	| selector |
	selector := spec getIndex.
	index := selector
		ifNil: [self list indexOf: (self model perform: spec getSelected)]
		ifNotNil: [spec model perform: selector]
! !

!ListStub methodsFor: 'events' stamp: 'cwp 5/26/2005 08:32'!
refreshList
	list := self model perform: spec list! !

!ListStub methodsFor: 'simulating' stamp: 'cwp 5/27/2005 08:22'!
selectedIndex
	^ index ifNil: [0]! !

!ListStub methodsFor: 'simulating' stamp: 'cwp 5/27/2005 08:27'!
selectedItem
	| items idx |
	(items  := self list) isEmpty ifTrue: [^ nil].
	(idx := self selectedIndex) = 0 ifTrue: [^ nil].
	^ items at: idx
	! !

!ListStub methodsFor: 'events' stamp: 'cwp 5/26/2005 08:57'!
update: aSelector
	aSelector = spec list ifTrue: [^ self refreshList].
	aSelector = spec getSelected ifTrue: [^ self refreshIndex].
	aSelector = spec getIndex ifTrue: [^ self refreshIndex].
	^ super update: aSelector! !

WidgetStub subclass: #MenuStub
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolBuilder-SUnit'!

!MenuStub methodsFor: 'as yet unclassified' stamp: 'cwp 7/14/2006 10:43'!
click: aString
	| item |
	item := self items detect: [:ea | ea label = aString] ifNone: [^ self].
	item action isSymbol
		ifTrue: [self model perform: item action]
		ifFalse: [item action value]! !

!MenuStub methodsFor: 'as yet unclassified' stamp: 'cwp 6/9/2005 00:29'!
items
	^ spec items! !

!MenuStub methodsFor: 'as yet unclassified' stamp: 'cwp 6/9/2005 00:36'!
labels
	^ self items keys! !

WidgetStub subclass: #TextStub
	instanceVariableNames: 'text'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolBuilder-SUnit'!

!TextStub methodsFor: 'simulating' stamp: 'cwp 4/22/2005 22:49'!
accept: aString
	^ self model perform: spec setText with: aString asText! !

!TextStub methodsFor: 'simulating' stamp: 'cwp 4/22/2005 22:43'!
color
	^ self model perform: spec color! !

!TextStub methodsFor: 'events' stamp: 'cwp 5/30/2005 23:11'!
eventAccessors
	^ #(setText selection menu color)! !

!TextStub methodsFor: 'events' stamp: 'cwp 5/30/2005 23:10'!
refresh
	self refreshText! !

!TextStub methodsFor: 'events' stamp: 'stephaneducasse 2/3/2006 22:32'!
refreshText
	text := self model perform: spec getText! !

!TextStub methodsFor: 'simulating' stamp: 'cwp 5/30/2005 23:11'!
text
	^ text! !

!TextStub methodsFor: 'events' stamp: 'cwp 5/30/2005 23:11'!
update: aSymbol
	aSymbol = spec getText ifTrue: [^ self refreshText].
	super update: aSymbol! !

WidgetStub subclass: #TreeNodeStub
	instanceVariableNames: 'item'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolBuilder-SUnit'!

!TreeNodeStub class methodsFor: 'instance creation' stamp: 'cwp 4/23/2005 00:15'!
fromSpec: aSpec item: anObject
	^ self new setSpec: aSpec item: anObject! !

!TreeNodeStub methodsFor: 'simulating' stamp: 'cwp 4/23/2005 00:50'!
children
	^ (self model perform: spec getChildren with: item)
		collect: [:ea | TreeNodeStub fromSpec: spec item: ea]! !

!TreeNodeStub methodsFor: 'simulating' stamp: 'cwp 4/23/2005 00:49'!
item
	^ item! !

!TreeNodeStub methodsFor: 'simulating' stamp: 'cwp 4/23/2005 00:57'!
label
	^ self model perform: spec label with: item! !

!TreeNodeStub methodsFor: 'private' stamp: 'cwp 4/23/2005 00:56'!
matches: aString
	^ self label = aString! !

!TreeNodeStub methodsFor: 'events' stamp: 'stephaneducasse 2/3/2006 22:32'!
openPath: anArray
	| child |
	anArray isEmpty 
		ifTrue: [self select]
		ifFalse: [child := self children 
								detect: [:ea | ea matches: anArray first] 
								ifNone: [^ self select].
				child openPath: anArray allButFirst]
	! !

!TreeNodeStub methodsFor: 'printing' stamp: 'cwp 4/23/2005 00:54'!
printOn: aStream
	aStream
		print: self class;
		nextPut: $<;
		print: item;
		nextPut: $>! !

!TreeNodeStub methodsFor: 'simulating' stamp: 'cwp 4/23/2005 00:23'!
select
	self model perform: spec setSelected with: item! !

!TreeNodeStub methodsFor: 'private' stamp: 'stephaneducasse 2/3/2006 22:32'!
selectPath: anArray
	| child |
	anArray isEmpty ifTrue: [^ self select].
	child := self children detect: [:ea | ea matches: anArray first] ifNone: [^ self select].
	child selectPath: anArray allButFirst.! !

!TreeNodeStub methodsFor: 'initialization' stamp: 'stephaneducasse 2/3/2006 22:32'!
setSpec: aSpec item: anObject
	super setSpec: aSpec.
	item := anObject! !

WidgetStub subclass: #TreeStub
	instanceVariableNames: 'roots'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolBuilder-SUnit'!

!TreeStub methodsFor: 'events' stamp: 'cwp 4/22/2005 22:40'!
eventAccessors
	^ #(roots getSelectedPath setSelected getChildren hasChildren label icon help menu keyPress)! !

!TreeStub methodsFor: 'private' stamp: 'stephaneducasse 2/3/2006 22:32'!
openPath: anArray
	| first |
	first := roots detect: [:ea | ea matches: anArray first] ifNone: [^ self].
	first openPath: anArray allButFirst! !

!TreeStub methodsFor: 'private' stamp: 'stephaneducasse 2/3/2006 22:32'!
roots: anArray
	roots := anArray collect: [:ea | TreeNodeStub fromSpec: spec item: ea].
! !

!TreeStub methodsFor: 'simulating' stamp: 'cwp 6/7/2005 00:31'!
select: anArray
	self openPath: anArray! !

!TreeStub methodsFor: 'initialization' stamp: 'cwp 4/23/2005 00:28'!
setSpec: aSpec
	super setSpec: aSpec.
	self update: spec roots! !

!TreeStub methodsFor: 'events' stamp: 'cwp 4/23/2005 01:00'!
update: anObject
	anObject == spec roots ifTrue: [^ self updateRoots].
	anObject == spec getSelectedPath ifTrue: [^ self updateSelectedPath].
	(anObject isKindOf: Array) ifTrue: [^ self openPath: anObject allButFirst].
	super update: anObject
	! !

!TreeStub methodsFor: 'events' stamp: 'cwp 4/23/2005 01:00'!
updateRoots
	^ self roots: (self model perform: spec roots)
! !

!TreeStub methodsFor: 'events' stamp: 'stephaneducasse 2/3/2006 22:32'!
updateSelectedPath
	| path first |
	path := self model perform: spec getSelectedPath.
	first := roots detect: [:ea | ea item = path first] ifNone: [^ self].
	first selectPath: path allButFirst.! !

!WidgetStub class methodsFor: 'instance creation' stamp: 'cwp 4/22/2005 21:41'!
fromSpec: aSpec
	^ self new setSpec: aSpec! !

!WidgetStub methodsFor: 'events' stamp: 'cwp 4/22/2005 22:01'!
eventAccessors
	^ #()! !

!WidgetStub methodsFor: 'simulating' stamp: 'cwp 4/22/2005 22:30'!
model
	^ spec model! !

!WidgetStub methodsFor: 'accessing' stamp: 'cwp 5/26/2005 08:48'!
name
	^ spec name ifNil: [' ']! !

!WidgetStub methodsFor: 'printing' stamp: 'cwp 4/25/2005 03:56'!
printOn: aStream
	aStream
		print: self class;
		nextPut: $<;
		nextPutAll: self name;
		nextPut: $>! !

!WidgetStub methodsFor: 'events' stamp: 'cwp 5/26/2005 08:39'!
refresh! !

!WidgetStub methodsFor: 'initialization' stamp: 'stephaneducasse 2/3/2006 22:32'!
setSpec: aSpec
	spec := aSpec.
	spec model addDependent: self.
	self refresh.! !

!WidgetStub methodsFor: 'accessing' stamp: 'cwp 4/25/2005 04:41'!
spec
	^ spec! !

!WidgetStub methodsFor: 'events' stamp: 'stephaneducasse 2/3/2006 22:32'!
update: aSelector
	| recognized |
	recognized := self eventAccessors collect: [:ea | spec perform: ea].
	(recognized includes: aSelector)
		ifTrue: [spec model perform: aSelector]! !

!WidgetStub methodsFor: 'accessing' stamp: 'cwp 4/25/2005 03:57'!
widgetNamed: aString
	^ self name = aString 
		ifTrue: [self]
		ifFalse: [nil]! !
